window.requestAnimFrame = (function(){
    return  window.requestAnimationFrame || 
    window.webkitRequestAnimationFrame || 
    window.mozRequestAnimationFrame    || 
    window.oRequestAnimationFrame      || 
    window.msRequestAnimationFrame     ||
    function( callback ){
        window.setTimeout(callback, 1000 / 60);
    };
})();

Math.rand = function(a, b) {
  return Math.random()*(b+1)|(a);
};

Kpong = {
	Param:  {
		ballR: 5,
		ballVx: 5,
		ballVy: 5,
		ballX: 100,
		ballY: 100,
		ballDx: 1,
		ballDy: 1,
		paddleH: 40,
		paddleY: 100,
		paddleVy: 5,
		score: 0,
		paddleW: 10,
		key: {}
	},
	init: function(id) {
		this.canvas = document.getElementById(id),
		this.ctx = this.canvas.getContext('2d'),
		this.w = this.canvas.width,
		this.h = this.canvas.height,
		this.paddleL =  new this.Paddle(0, this.Param);
		this.paddleR =  new this.Paddle(this.w-10, this.Param);
		this.ball =  new this.Ball(this.Param);
		this.audioTable = new Audio("table.ogg");
		this.audioPing = new Audio("ping.ogg");
		this.audioPong = new Audio("pong.ogg");
		this.addEvents(this.Param.key, this.paddleL);
		this.main();
	},
	Paddle: function(x, param) {
		this.x = x, 
		this.y = param.paddleY,
		this.vy = param.paddleVy,
		this.w = param.paddleW,
		this.h = param.paddleH, 		
		this.minY = 0,
		this.maxY = param.h - param.paddleH,
		this.s = param.score
		return this;
	},
	Ball: function(param) {
		this.x = param.ballX, 
		this.y = param.ballY,
		this.r = param.ballR,
		this.vx = param.ballVx,
		this.vy = param.ballVy,
		this.dx = param.ballDx,
		this.dy = param.ballDy
		return this;
	},
	addEvents: function(key, paddleL) {
		window.addEventListener("keydown", function(e){
			key[e.keyCode || e.which] = true;
		});

		window.addEventListener("keyup", function(e){
			key[e.keyCode || e.which] = false;
		});
		
		document.getElementById('game').addEventListener("touchstart", function(e) {
			var y = e.touches[0].clientY;
			paddleL.y = y-paddleL.h/2;
			
			
		}, false);
	},
	keydown: function() {
		if(this.Param.key[40] && this.paddleL.y+this.paddleL.h < this.h) 
			this.paddleL.y += 8;
		else if(this.Param.key[38] && this.paddleL.y > 0) 
			this.paddleL.y -= 8;
	},
	main: function() {
		this.keydown();
		this.deplacementBall(this.ball);
		this.ai(this.ball, this.paddleR);
		this.collision(this.ball, [this.paddleL, this.paddleR]);
		this.draw();
		requestAnimFrame(function(){
			Kpong.main();
		});
	},
	draw: function() {
		this.ctx.clearRect(0, 0, this.w, this.h);
		this.drawScore(this.ctx, this.paddleL.s, this.paddleR.s);
		this.drawPaddle(this.ctx, this.paddleL, this.paddleR);
		this.drawBall(this.ctx, this.ball);
	},
	collision: function(ball, paddle) {

		//Bords verticaux
		if(ball.x+ball.r >= this.w)
		{
			this.paddleL.s++;
			ball.x = this.w/2;
			ball.y = this.h/2;
			//this.paddleR.y = this.h/2;
			ball.dy = 0;
			ball.vx = 3;
		}
		else if(ball.x-ball.r <= 0)
		{
			this.paddleR.s++;
			ball.x = this.w/2;
			ball.y = this.h/2;
			ball.dy = 0;
			ball.vx = 3;
		}

		//Bords horizontaux
		if(ball.y-ball.r <= 0 || ball.y+ball.r >= this.h)
		{
			ball.dy *= -1;
			this.audioTable.play();
		}
			

		// Si la balle touche une raquete
		for(i=0;i<paddle.length;i++)
		{
			if((ball.y+ball.r >= paddle[i].y && ball.y-ball.r <= paddle[i].y+paddle[i].h) && ball.x-ball.r-ball.vx < paddle[i].x+paddle[i].w && ball.x+ball.r+ball.vx > paddle[i].x)
			{
				
				var dist1 = paddle[i].x+paddle[i].w - ball.x-ball.r;
				var dist2 = ball.x+ball.r - paddle[i].x;
				if(dist1 < dist2)
				{
					ball.x = paddle[i].x+paddle[i].w+ball.r;
					this.audioPing.play();
				
				}
				else
				{
					ball.x = paddle[i].x-ball.r;
					this.audioPong.play();
				}
					
				ball.dx *= -1;
				if(ball.dy == 0) ball.dy = 1;
				
				if(Math.random()*2|0 == 0) 
					ball.vx++; 
				else
					ball.vy++; 
				
			}
		}
	},
	deplacementBall: function(ball) {
		ball.x += ball.vx*ball.dx;
		ball.y += ball.vy*ball.dy;
		if(ball.y <= 0) 
			ball.y = 0;
		else if(ball.y+ball.r >= this.h)
			ball.y = this.h-ball.r;
	},
	ai: function(ball, paddleR) {
		
		if(paddleR.y <= 6) 
			paddleR.y = 6;
		if(paddleR.y+paddleR.h >= this.h-6) 
			paddleR.y = this.h-paddleR.h-6;

		var difference = ball.y - paddleR.y;

		if((difference >= 5 || difference <= -5) && ball.dx == 1)
		{
			if(difference > 0)
			{
				paddleR.y+=ball.vy-Math.rand(0, 2);
			}
			else
			{
				paddleR.y-=ball.vy+Math.rand(0, 2);
			}
		}
	},
	drawPaddle: function(ctx, paddleL, paddleR) {
		ctx.shadowBlur=15;
		ctx.shadowColor = "#fff";
		ctx.fillStyle = "#333";
		ctx.beginPath();
			ctx.fillRect(paddleL.x, paddleL.y, paddleL.w, paddleL.h);
			ctx.fillRect(paddleR.x, paddleR.y, paddleR.w, paddleR.h);
		ctx.closePath();
		ctx.stroke();
		ctx.shadowBlur=0;
	},
	drawBall: function(ctx, ball) {
		ctx.fillStyle = "#fff200";
		ctx.beginPath();
			ctx.arc(ball.x, ball.y, ball.r, 0, 2*Math.PI);  
		ctx.closePath();
		ctx.fill();
	},
	drawScore: function(ctx, scoreL, scoreR) {
		ctx.fillStyle = "#FFF";
		ctx.font = "50px courier new";
		ctx.fillText(scoreL, this.w/2-60, 50);
		ctx.fillText(scoreR, this.w/2+30, 50);
	}
}

var teste = Kpong.init('game');